module.exports = function (grunt) {

	var ecommerce = {
		js: ["./assets/js/util/**/*.js", "./assets/js/repositories/**/*.js", "./assets/js/services/**/*.js", "./assets/js/controllers/**/*.js", "./assets/templates/js/*.js", "!./assets/js/libs.js", "!./assets/js/engine.js", "!./assets/js/controllers/LoginController.js", "!./assets/js/services/LoginService.js", "!./assets/js/util/LoginUtilities.js", "!./assets/js/environment.js", "!./assets/js/app.js", "!./assets/js/app.min.js"],
		css: ["./assets/css/**/*.css", "./assets/templates/css/*.css", "!./assets/css/app.css", "!./assets/css/app.min.css"],
		libs: [
			"./bower_components/zh-cache-bypass/src/CacheBypass.js",
			"./bower_components/zh-preferences/dist/preferences.js",
			"./bower_components/zh-filter-preferences/dist/filter-preferences.js",
			"./bower_components/zh-layout-preferences/dist/layout-preferences.js",
			"./bower_components/zh-grid-configuration/dist/zh-grid-configuration.js",
			"./bower_components/zh-view-preferences/dist/view-preferences.js",
			"./bower_components/zh-rich-text/dist/zh-rich-text.js",
			"./bower_components/zh-interval-component/dist/zh-interval-component.js",
			"./bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js",
			"./bower_components/zh-reports/dist/zhReports.js",
			"./bower_components/zhuaclfrontend/assets/dist/main.js",
			"./bower_components/zhulog/dist/main.js",
			'./bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js',
			"./bower_components/tek-libraries/dist/main.js"
		]
	};

	grunt.initConfig({
		concat: {
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: ecommerce.js,
				dest: 'assets/js/app.js'
			},
			libs: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: ecommerce.libs,
				dest: 'assets/js/libs.js'
			},
			css: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: ecommerce.css,
				dest: 'assets/css/app.css'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'assets/js/app.min.js': ecommerce.js
				}
			}
		},
		jshint: {
			all: ecommerce.js.concat(["GruntFile.js"]),
			options: {
				evil: true,
				esversion: 6
			}
		},
		watch: {
			js: {
				files: ecommerce.js.concat(["GruntFile.js"]),
				tasks: ['jshint', 'concat:js']
			},
			css: {
				files: ecommerce.css,
				tasks: ['concat:css']
			}
		},
		cssmin: {
			dist: {
				src: ecommerce.css,
				dest: 'assets/css/app.min.css'
			}
		},
		// NÂO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
		zhIdGenerator: {
			options: {
				jsonPath: ["./assets/json"],
				menuPath: ["./assets/json/menu.json"]
			}
		},
		buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
		buildEnvironment: {
			development: {
				configFile: "./config/devEnvironment.json"
			},
			production: {
				configFile: "./config/prodEnvironment.json"
			},
			test: {
				configFile: "./config/testEnvironment.json"
			},
			srcFolders: ["../backend/config/", "./assets/js", "./assets/json", "./config", "./", "./assets", "../backend/reports"]
		},
		zhWords: {
			options: {
				serverUrl: "https://translate.teknisa.com/backend/index.php",
				organization: "20",
				project: "3317"
			}
		}
	});
	
	function inArray(array, value) {
        return array.filter(function(position) {
            return position == value;
        }).length;
    }
	
	function buildFileContent(srcFiles, filePath, directory) {
        var parts = filePath.split('/');
        parts.shift();
        parts.shift();
        var filename = parts[parts.length - 1];
        var extension = filename.split('.').pop().toUpperCase();
        var files = ['routes', 'controllers', 'services','util'];
        if (extension == 'JSON' || extension == 'XML') {
            if (directory == 'datasources') {
                filename = parts.pop();
                filename = filename.replace(/\.[^/.]+$/, "");
                if (filePath.search('backend') != -1) {
                    srcFiles[directory][filename] = './../' + parts.join('/');
                } else {
                    srcFiles[directory][filename] = './../../modules/' + parts.join('/');
                }
            } else {
                if (inArray(files, filename.split('.')[0])) {
                    srcFiles[directory].push('/' + parts.join('/'));
                }
            }
        }
    }
	
	grunt.registerTask('backendStep', 'build backend config files', function () {
		var srcFiles = {
			config: [],
			datasources: {}
		};
		var directories = ['config', 'datasources'];
		var dest = {
			config: '../backend/config/servicesMapping.json',
			datasources: '../backend/config/dataSourcesMapping.json'
		};

		directories.forEach(function (directory) {
            grunt.file.expand({
                filter: 'isFile'
            }, '../backend/src/Ecommerce/' + directory + '/**/*').forEach(function (filePath) {
                buildFileContent(srcFiles, filePath, directory);
            });
		});

		grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
	});
	
	grunt.registerTask('checkInternalWords', 'Check and remove duplicate values', function() {
		try {
			const pathInternalWords = 'assets/json/internalWords.json';
			let internalWords = grunt.file.readJSON(pathInternalWords);
			
			if (!internalWords.words) {
				throw "The 'words' property was not found.";
			}
		
			internalWords.words = [...new Set(internalWords.words)];
		
			grunt.file.write(pathInternalWords , JSON.stringify(internalWords, null, 4));
		} catch (err) {
			grunt.fail.fatal(err);
		}
	});

	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-cssmin');
	grunt.loadNpmTasks('grunt-zh-id-generator');
	grunt.loadNpmTasks('grunt-zhuacl-structure');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-words');
		
	grunt.registerTask('default', ['zhWords','jshint', 'backendStep', 'concat', 'buildAclStruct', 'buildEnvironment']);
	grunt.registerTask('cloud9', ['jshint', 'backendStep', 'concat', 'buildAclStruct', 'buildEnvironment']);
	grunt.registerTask('dev', ['backendStep', 'concat', 'buildEnvironment']);
	grunt.registerTask('mergeBranch', ['zhIdGenerator', 'buildAclStruct']);
	grunt.registerTask('final', ['cssmin', 'uglify']);
};